@@ -21,8 +21,10 @@
 
 local jsonschema = require('jsonschema')
 local lrucache = require("apisix.core.lrucache")
+local schema_def = require("apisix.schema_def")
 local cached_validator = lrucache.new({count = 1000, ttl = 0})
 local pcall = pcall
+local error = error
 
 local _M = {
     version = 0.3,
@@ -68,4 +70,9 @@ end
 
 _M.valid = get_validator
 
+setmetatable(_M, {
+    __index = schema_def,
+    __newindex = function() error("no modification allowed") end,
+})
+
 return _M
