@@ -25,32 +25,10 @@ local ngx          = ngx
 local core         = require("apisix.core")
 
 local _M = {
-    events_module = nil,
 }
 
-_M.EVENTS_MODULE_LUA_RESTY_WORKER_EVENTS = 'lua-resty-worker-events'
-_M.EVENTS_MODULE_LUA_RESTY_EVENTS = 'lua-resty-events'
-
-
--- use lua-resty-worker-events
-local function init_resty_worker_events()
-    _M.events_module = _M.EVENTS_MODULE_LUA_RESTY_WORKER_EVENTS
-
-    local we = require("resty.worker.events")
-    local shm = ngx.config.subsystem == "http" and "worker-events" or "worker-events-stream"
-    local ok, err = we.configure({shm = shm, interval = 0.1})
-    if not ok then
-        error("failed to init worker event: " .. err)
-    end
-
-    return we
-end
-
-
 -- use lua-resty-events
 local function init_resty_events()
-    _M.events_module = _M.EVENTS_MODULE_LUA_RESTY_EVENTS
-
     local listening = "unix:" .. ngx.config.prefix() .. "logs/"
     if ngx.config.subsystem == "http" then
         listening = listening .. "worker_events.sock"
@@ -81,19 +59,7 @@ function _M.init_worker()
     end
 
     _M.inited = true
-
-    local conf = core.config.local_conf()
-    local module_name = core.table.try_read_attr(conf, "apisix", "events", "module")
-                            or _M.EVENTS_MODULE_LUA_RESTY_WORKER_EVENTS
-
-    if module_name == _M.EVENTS_MODULE_LUA_RESTY_EVENTS then
-        -- use lua-resty-events as an event module via the apisix.events.module
-        -- key in the configuration file
-        _M.worker_events = init_resty_events()
-    else
-        -- use lua-resty-worker-events default now
-        _M.worker_events = init_resty_worker_events()
-    end
+    _M.worker_events = init_resty_events()
 end
 
 
@@ -105,20 +71,15 @@ end
 function _M.event_list(self, source, ...)
     -- a patch for the lua-resty-events to support event_list
     -- this snippet is copied from the lua-resty-worker-events lib
-    if self.events_module == _M.EVENTS_MODULE_LUA_RESTY_EVENTS then
-        local events = { _source = source }
-        for _, event in pairs({...}) do
-            events[event] = event
-        end
-        return setmetatable(events, {
-            __index = function(_, key)
-            error("event '"..tostring(key).."' is an unknown event", 2)
-            end
-        })
+    local events = { _source = source }
+    for _, event in pairs({...}) do
+        events[event] = event
     end
-
-    -- the lua-resty-worker-events has a built-in event_list implementation
-    return self.worker_events.event_list(source, ...)
+    return setmetatable(events, {
+        __index = function(_, key)
+        error("event '"..tostring(key).."' is an unknown event", 2)
+        end
+    })
 end
 
 
@@ -127,13 +88,4 @@ function _M.post(self, ...)
 end
 
 
-function _M.get_healthcheck_events_module(self)
-    if self.events_module == _M.EVENTS_MODULE_LUA_RESTY_EVENTS then
-        return "resty.events"
-    else
-        return "resty.worker.events"
-    end
-end
-
-
 return _M
