@@ -59,6 +59,10 @@ local expr_lrucache = core.lrucache.new({
 local meta_pre_func_load_lrucache = core.lrucache.new({
     ttl = 300, count = 512
 })
+local merge_global_rule_lrucache = core.lrucache.new({
+    ttl = 300, count = 512
+})
+
 local local_conf
 local check_plugin_metadata
 
@@ -753,12 +757,6 @@ function _M.merge_consumer_route(route_conf, consumer_conf, consumer_group_conf,
     local new_conf = merged_route(flag, api_ctx.conf_version,
                         merge_consumer_route, route_conf, consumer_conf, consumer_group_conf)
 
-    -- some plugins like limit-count don't care if consumer changes
-    -- all consumers should share the same counter
-    api_ctx.conf_type_without_consumer = api_ctx.conf_type
-    api_ctx.conf_version_without_consumer = api_ctx.conf_version
-    api_ctx.conf_id_without_consumer = api_ctx.conf_id
-
     api_ctx.conf_type = api_ctx.conf_type .. "&consumer"
     api_ctx.conf_version = api_ctx.conf_version .. "&" ..
                            api_ctx.consumer_ver
@@ -944,6 +942,7 @@ local function enable_gde()
 
     return enable_data_encryption
 end
+_M.enable_gde = enable_gde
 
 
 local function get_plugin_schema_for_gde(name, schema_type)
@@ -1262,7 +1261,45 @@ function _M.set_plugins_meta_parent(plugins, parent)
 end
 
 
-function _M.run_global_rules(api_ctx, global_rules, phase_name)
+local function merge_global_rules(global_rules, conf_version)
+    -- First pass: identify duplicate plugins across all global rules
+    local plugins_hash = {}
+    local seen_plugin = {}
+    local values = global_rules
+    for _, global_rule in config_util.iterate_values(values) do
+        if global_rule.value and global_rule.value.plugins then
+            for plugin_name, plugin_conf in pairs(global_rule.value.plugins) do
+                if seen_plugin[plugin_name] then
+                    core.log.error("Found ", plugin_name,
+                                  " configured across different global rules.",
+                                  " Removing it from execution list")
+                    plugins_hash[plugin_name] = nil
+                else
+                    plugins_hash[plugin_name] = plugin_conf
+                    seen_plugin[plugin_name] = true
+                end
+            end
+        end
+    end
+
+    local dummy_global_rule = {
+        key = "/apisix/global_rules/dummy",
+        value = {
+            updated_time = ngx.time(),
+            plugins = plugins_hash,
+            created_time = ngx.time(),
+            id = 1,
+        },
+        createdIndex = conf_version,
+        modifiedIndex = conf_version,
+        clean_handlers = {},
+    }
+
+    return dummy_global_rule
+end
+
+
+function _M.run_global_rules(api_ctx, global_rules, conf_version, phase_name)
     if global_rules and #global_rules > 0 then
         local orig_conf_type = api_ctx.conf_type
         local orig_conf_version = api_ctx.conf_version
@@ -1272,22 +1309,26 @@ function _M.run_global_rules(api_ctx, global_rules, phase_name)
             api_ctx.global_rules = global_rules
         end
 
-        local plugins = core.tablepool.fetch("plugins", 32, 0)
-        local values = global_rules
-        local route = api_ctx.matched_route
-        for _, global_rule in config_util.iterate_values(values) do
-            api_ctx.conf_type = "global_rule"
-            api_ctx.conf_version = global_rule.modifiedIndex
-            api_ctx.conf_id = global_rule.value.id
+        local dummy_global_rule = merge_global_rule_lrucache(conf_version,
+                                                             global_rules,
+                                                             merge_global_rules,
+                                                             global_rules,
+                                                             conf_version)
 
-            core.table.clear(plugins)
-            plugins = _M.filter(api_ctx, global_rule, plugins, route)
-            if phase_name == nil then
-                _M.run_plugin("rewrite", plugins, api_ctx)
-                _M.run_plugin("access", plugins, api_ctx)
-            else
-                _M.run_plugin(phase_name, plugins, api_ctx)
-            end
+        local plugins = core.tablepool.fetch("plugins", 32, 0)
+        local route = api_ctx.matched_route
+        api_ctx.conf_type = "global_rule"
+        api_ctx.conf_version = dummy_global_rule.modifiedIndex
+        api_ctx.conf_id = dummy_global_rule.value.id
+
+        core.table.clear(plugins)
+        plugins = _M.filter(api_ctx, dummy_global_rule, plugins, route)
+
+        if phase_name == nil then
+            _M.run_plugin("rewrite", plugins, api_ctx)
+            _M.run_plugin("access", plugins, api_ctx)
+        else
+            _M.run_plugin(phase_name, plugins, api_ctx)
         end
         core.tablepool.release("plugins", plugins)
 
