@@ -19,6 +19,8 @@ local ngx = ngx
 local ngx_re = require("ngx.re")
 local consumer_mod = require("apisix.consumer")
 local ldap = require("resty.ldap")
+local schema_def = require("apisix.schema_def")
+
 
 local schema = {
     type = "object",
@@ -28,7 +30,8 @@ local schema = {
         ldap_uri = { type = "string" },
         use_tls = { type = "boolean", default = false },
         tls_verify = { type = "boolean", default = false },
-        uid = { type = "string", default = "cn" }
+        uid = { type = "string", default = "cn" },
+        realm = schema_def.get_realm_schema("ldap"),
     },
     required = {"base_dn","ldap_uri"},
 }
@@ -106,7 +109,7 @@ function _M.rewrite(conf, ctx)
     -- 1. extract authorization from header
     local auth_header = core.request.header(ctx, "Authorization")
     if not auth_header then
-        core.response.set_header("WWW-Authenticate", "Basic realm='.'")
+        core.response.set_header("WWW-Authenticate", "Basic realm=\"" .. conf.realm .. "\"")
         return 401, { message = "Missing authorization in request" }
     end
 
@@ -117,6 +120,7 @@ function _M.rewrite(conf, ctx)
         else
           core.log.warn("nil user")
         end
+        core.response.set_header("WWW-Authenticate", "Basic realm=\"" .. conf.realm .. "\"")
         return 401, { message = "Invalid authorization in request" }
     end
 
@@ -136,6 +140,7 @@ function _M.rewrite(conf, ctx)
     local res, err = ldap.ldap_authenticate(user.username, user.password, ldapconf)
     if not res then
         core.log.warn("ldap-auth failed: ", err)
+        core.response.set_header("WWW-Authenticate", "Basic realm=\"" .. conf.realm .. "\"")
         return 401, { message = "Invalid user authorization" }
     end
 
@@ -144,12 +149,14 @@ function _M.rewrite(conf, ctx)
     -- 3. Retrieve consumer for authorization plugin
     local consumer_conf = consumer_mod.plugin(plugin_name)
     if not consumer_conf then
+        core.response.set_header("WWW-Authenticate", "Basic realm=\"" .. conf.realm .. "\"")
         return 401, { message = "Missing related consumer" }
     end
 
     local consumers = consumer_mod.consumers_kv(plugin_name, consumer_conf, "user_dn")
     local consumer = consumers[user_dn]
     if not consumer then
+        core.response.set_header("WWW-Authenticate", "Basic realm=\"" .. conf.realm .. "\"")
         return 401, {message = "Invalid user authorization"}
     end
     consumer_mod.attach_consumer(ctx, consumer, consumer_conf)
