@@ -191,7 +191,7 @@ function _M.rewrite(conf, ctx)
 
     local proxy_proto = core.request.header(ctx, "X-Forwarded-Proto")
     local _scheme = proxy_proto or core.request.get_scheme(ctx)
-    if conf.http_to_https and _scheme == "http" then
+    if conf.http_to_https and _scheme ~= "https" then
         if ret_port == nil or ret_port == 443 or ret_port <= 0 or ret_port > 65535  then
             uri = "https://$host$request_uri"
         else
